--[[

# Utilities.lua

Contains utilities that may be useful across multiple addons and addon 
modules. This module should have no additional lua dependencies, in order to 
avoid circular dependencies.

## .NET Library Requirements

- log4net

]]

luanet.load_assembly("System")
luanet.load_assembly("log4net")

Utilities = {}
Utilities.__index = Utilities

Utilities.Log = luanet.import_type("log4net.LogManager").GetLogger("AtlasSystems.Addons.Utilities")

function Utilities:IsType(o, t)
    --[[

        Determines if the specified object "o" is of .NET type "t". t should 
        be a string of the full name of the .NET type, such as 
        "System.DateTime". 

    ]]
    local Log = Utilities.Log

    if ((o and type(o) == "userdata") and (t and type(t) == "string")) then
        local comparisonType = Types["System.Type"].GetType(t)

        Log:DebugFormat("[IsType] Evaluating Type: {0}", o:GetType().FullName)

        if (comparisonType) then
            -- The comparison type was successfully loaded so we can do a check
            -- that the object can be assigned to the comparison type.
            return comparisonType:IsAssignableFrom(o:GetType()), true
        else
            -- The comparison type was could not be loaded so we can only check
            -- based on the names of the types.
            return (o:GetType().FullName == t), false
        end
    end

    return false, false
end


function Utilities:LogTable (luaTable, stringPrefix)
    --[[

        Writes the contents of a table or array to the log. The string prefix 
        is written before each key. 

    ]]
    local Log = Utilities.Log    

    stringPrefix = stringPrefix or ""

    if type(luaTable) ~= "table" then
        Log:WarnFormat("[LogTable] Input was not a table. Instead was of type '{0}'", type(luaTable))
        return
    end

    for key,value in pairs(luaTable) do
        if type(value) == "table" then
            Utilities:LogTable(value, string.format("%s%s.", stringPrefix, key))
        else
            Log:DebugFormat("{2}{0} = {1}", key, value, stringPrefix)
        end
    end
end
